#include <fcntl.h>
#include <sys/mman.h>
#include <stdio.h>

#include <new>

#include <SupportDefs.h>

#include <private/shared/AutoDeleter.h>


class Block
{
public:
	Block *next;
	HandleDeleter<int, int, close> file;
	void *ptr;
	size_t size;

	Block(const char *name):
		file(open(name, O_RDONLY)),
		ptr(NULL)
	{
		if (file.Get() >= 0) {
			struct stat s;
			fstat(file.Get(), &s);
			size = s.st_size;
			ptr = mmap(0, s.st_size, PROT_READ, MAP_PRIVATE, file.Get(), 0);
		}
	}

	~Block()
	{
		munmap(ptr, size);
		ptr = NULL;
	}

};


int main()
{
	Block *blocks = NULL;
	size_t totalSize = 0;

	for (int i = 0; i < 4096; i++) {
		Block *blk = new(std::nothrow) Block("TestFile");
		if (blk == NULL || blk->ptr == NULL)
			break;
		blk->next = blocks; blocks = blk;
		totalSize += blk->size;
	}

	printf("Allocated: %" B_PRIuSIZE, totalSize); getc(stdin);

	while (blocks != NULL) {
		Block *next = blocks->next;
		totalSize -= blocks->size;
		delete blocks;
		blocks = next;
	}

	printf("Freed: %" B_PRIuSIZE, totalSize); getc(stdin);

	return 0;
}
